<?php

namespace App;

use Nette\Security\Passwords;

class UserEditForm extends DefaultForm {

	/** @var array */
	public $onClose;
	
	public function __construct(IContainer $parent = NULL, $name = NULL) 
	{
		parent::__construct($parent, $name);
		// $this->jsHelper = 'person';
		$this->addHidden('id');
		$this->addText("name", "Jméno")->setRequired("Zadejte jméno");
		$this->addText("username", "Uživatelské jméno")->setRequired("Zadejte uživatelské jméno");
		$this->addText("code", "Kód")
			->addCondition(self::FILLED)
			->addRule(self::PATTERN, 'Chybný kód karty', Glob::$codeRE);
		$this->addPassword('password', 'Heslo');
		$this->addPassword('passwordVerify', 'Heslo pro kontrolu')
			->setRequired(false)
			->addRule(self::EQUAL, 'Hesla se neshodují', $this['password']);
		$this->addSelect('roleId', 'Role', Role::findAll()->selection->fetchPairs('id', 'name'));
		$this->addSubmit('btnSave', 'Uložit')->onClick[] = [$this, 'formSave'];
		$this->addSubmit('btnCancel', 'Storno')
			->setValidationScope(null)
			->onClick[] = [$this, 'formCancel'];
	}
	
	public function loadItem($id) 
	{
		if ((int)$id) {
			unset($this['btnDelete']);
			$this['password']
				->addCondition(self::FILLED)
				->addRule(self::MIN_LENGTH, 'Heslo musí mít alespoň %d znaky', 3);
		} else {
			$this['password']
				->setRequired('Zvolte si heslo')
				->addRule(self::MIN_LENGTH, 'Heslo musí mít alespoň %d znaky', 3);
		}
		$id = (int)$id;
		$defaults = $id
			? User::get($id)->toArray()
			: array(); // User::defaults();
		$this->setDefaults(
			$defaults
		);
	}
	
	public function formSave($button) 
	{
		$values = $button->form->values;//->toArray();
		if ($values->password) {
			$values->passwordHash = Passwords::hash($values->password); // UserManager::calculateHash($values->password, $values->username);
		}
		unset($values->password);
		unset($values->passwordVerify);

		$id = (int)$values->id; 
		$user = $id
			? User::get($id)
			: new User();
		$user->fromArray($values);
		$user->save();
		$this->onClose();
	}

	public function formCancel($button) 
	{
		$this->onClose();
	}
	
	public function formDelete($button) 
	{
		
	}
}