<?php

namespace App;
/** 
@dbTable
	@property autoIncrement $id
	@property float $currentAmount
	@property float $changeAmount
	@property ProductSell(fk,?) $productSell
	@property Product(fk) $product
	@property StockFill(fk,?) $stockFill
	@property int $utChange
	@property bool[,,true] $isActual
**/

class StockChange extends BaseModel 
{
	public function save()
	{
		if ($this->_dbEntity->isNew())
		{
			if ($this->isActual) 
			{
				$q = self::repository()->newSqlQuery()
					->code('START TRANSACTION;')
					// Zjisti aktuální počet
					->code('SELECT @newAmount := IFNULL((SELECT currentAmount FROM stockChange WHERE productId=')
						->value($this->productId)->code('AND isActual=1), 0) + ')->value($this->changeAmount)->code(';')
					// Označ aktuální jako už neaktuální
					->code('UPDATE stockChange SET isActual=0 WHERE productId=')
						->value($this->productId)->code(' AND isActual=1;')
					// Vlož nový
					->code('INSERT INTO stockChange (currentAmount, changeAmount, productId, productSellId, stockFillId, utChange, isActual) VALUES (')
						->code('@newAmount,')
						->values([
							$this->changeAmount,
							$this->productId,
							$this->productSellId,
							$this->stockFillId,
							$this->utChange,
							$this->isActual
						])
					->code('); COMMIT')
					->exec();
			} //else parent::save();
		} else parent::save();
	}

	public function getChangeType()
	{
		if ($this->productSellId)
			return $this->changeAmount > 0 ? 'cancel' : 'sell';
		else if ($this->stockFillId)
			return $this->changeAmount > 0 ? 'stockFill' : 'stockOut';
		return 'unknown';
	}

	public static $typeTitle = [
		'cancel' => 'storno',
		'sell' => 'prodej',
		'stockFill' => 'naskladnění',
		'stockOut' => 'vyskladnění',
		'unknown' => '-'
	];

	public function getChangeTypeTitle()
	{
		return self::$typeTitle[$this->changeType];
	}

	// Repo
		public static function productStat($utFrom, $utTo)
		{
			$sel = self::repository()->newSqlQuery()
				->code('SELECT p.name AS productName, p.id AS productId, sch.utChange AS utLastChange, sch.currentAmount, sch.changeAmount AS lastChange, 
					SUM(IF(ss.productSellId IS NOT NULL AND ss.changeAmount > 0, ss.changeAmount, 0)) AS sellIn, 
					ABS(SUM(IF(ss.productSellId IS NOT NULL AND ss.changeAmount < 0, ss.changeAmount, 0))) AS sellOut,
					SUM(IF(ss.stockFillId IS NOT NULL AND ss.changeAmount > 0, ss.changeAmount, 0)) AS fillIn, 
					ABS(SUM(IF(ss.stockFillId IS NOT NULL AND ss.changeAmount < 0, ss.changeAmount, 0))) AS fillOut
					FROM product p
					LEFT JOIN stockChange sch ON sch.productId = p.id AND sch.isActual=1
					LEFT JOIN stockChange ss ON ss.productId = p.id ');
			if ($utFrom) $sel->code(' AND ss.utChange >=')->value($utFrom); 
			if ($utTo) $sel->code(' AND ss.utChange <=')->value($utTo); 
			$sel->code('WHERE p.stockable = 1 GROUP BY p.id');

			return $sel->getAsRows();
		}

		public static function productHistory($productId)
		{
			return self::findBy(['productId' => $productId])->order('utChange DESC');
		}

}
