<?php

namespace App;

use Nette\ComponentModel\IContainer;

class CustomerEditForm extends DefaultForm {

	/** @var array */
	public $onClose;
	
	/** @var array */
	public $onDelete;
	
	public function __construct(IContainer $parent = NULL, $name = NULL) {
		parent::__construct($parent, $name);
		// $this->jsHelper = 'person';
		$this->addHidden('id');
		$this->addText("nameFirst", "Jméno")->setRequired("Zadejte jméno");
		$this->addText("nameLast", "Příjmení")->setRequired("Zadejte příjmení");
		$this->addText("note", "Poznámka");
		$this->addText("code", "Kód")
			->addCondition(self::FILLED)
			->addRule(self::PATTERN, 'Chybný kód karty', Glob::$codeRE);
		$this->addText("contactPhone", "Telefon");
		$this->addText("contactAddress", "Adresa");
		// $form->addDate('createdFrom', 'Založeno', DateInput::TYPE_DATE);
		$this->addDateInputDT("dtBirth", "Datum narození");
		$this->addText("rc", "Rodné číslo");
        $this->addRadioList("memberLevelId", "Členství", MemberLevel::findAll()->fetchPairs("id", "name") + [null => '-']);
		$this->addSubmit('btnSave', 'Uložit')->onClick[] = [$this, 'formSave'];
		$this->addSubmit('btnCancel', 'Storno')
			->setValidationScope(null)
			->onClick[] = [$this, 'formCancel'];
		$this->onValidate[] = function($form) 
		{ 
			if (!Customer::isCodeUnique($form['code']->value, $form['id']->value))
				$form->addError("Kód " . $form['code']->value . ' už používá jiný zákazník.');
		};
		/* $this->addSubmit('btnDelete', 'Smazat')
			->setValidationScope(null)
			->onClick[] = $this->formDelete; */
	}
	
	public function loadItem($id) 
	{
		if (!(int)$id) unset($this['btnDelete']);
		$defaults = (int)$id
			? Customer::get($id)->toArray()
			: [];
		// bdump($defaults);
		$this->setDefaults(
			$defaults
		);
	}
	
	public function formSave($button) 
	{
		$values = $button->form->values;//->toArray();
		$id = (int)$values->id;
		$customer = $id
			? Customer::get($id)
			: new Customer();

		$customer->fromArray($values);
		$customer->save();

		$this->onClose();
	}

	public function formCancel($button) {
		$this->onClose();
	}
	
	public function formDelete($button) {
		$this->onDelete();
	}
}