<?php

namespace App;

/** 
 * @dbTable
 * @property int $id (autoIncrement)
 * @property string $name [200]
 * @property int $validDuration (?)
 * @property ?decimal $credit (?)
 * @property bool $deleted
 * @property bool $creditChargeable
 * @property bool $creditNegative
 * @property int $notifyBefore (?)
 * @property int $validDurationHint (?)
 * @related PackageProduct $packageProducts
 * @related Product $sellAsProducts
**/
class Package extends BaseModel 
{
	public $level = 0;

	static function defaults() {
		return array(
			"enabled" => true,
			"position" => 0,
			'validDuration' => null,
			'credit' => null,
			'notifyBefore' => NULL,
		);
	}
	
	public function getProducts() 
	{
		return $this->packageProducts->where('amount IS NOT NULL OR fromCredit OR priceDirect IS NOT NULL');
	}
	
	public function getSellAs() 
	{
		return $this->sellAsProducts->where(array('deleted' => 0));
	}
	
	public function makePrepaid($customerId) 
	{
		$prepaid = new Prepaid();
		$prepaid->utCreated = time();
		$prepaid->utValidFrom = $this->validDuration ? null : time();
		$prepaid->creditCALeft = (float)$this->credit;
        if ($this->validDurationHint) {
            $prepaid->utValidToHint = time() + $this->validDurationHint;
        }
		$prepaid->customerId = $customerId;
		$prepaid->packageId = $this->id;
		$prepaid->userId = Glob::currentAccountId();
		$prepaid->save();
		// Predplacene produkty
		foreach($this->packageProducts->where('amount != 0') as $prod) {
			$ppp = new PrepaidPackageProduct();
			$ppp->prepaidId = $prepaid->id;
			$ppp->packageProductId = $prod->id;
			$ppp->amountLeft = $prod->amount;

			$ppp->save();
		}
		
		return $prepaid; 
	}

	public static function formValues($id) {
		$dbo = self::get($id);
		$ret = $dbo->toArray();
		 /*array(
			'id' => $dbo['id'],
			'name' => $dbo['name'],
			'validDuration' => $dbo['validDuration'],
			'credit' => $dbo['credit'],
			'creditNegative' => $dbo['creditNegative'],
			'creditChargeable' => $dbo['creditChargeable'],
		);*/
		foreach(PackageProduct::findBy(array('packageId' => $id)) as $row) 
		{
			$ret['product'.$row->productId] = $row->toArray();
		}
		
		return $ret;
	}

}