<?php

namespace App;

use Nette\NObject;

/**
 * @dbTable
 * @property int $id (autoIncrement)
 * @property string $name [200]
 * @property string $description
 * @property decimal $price
 * @property bool $enabled [,,1]
 * @property bool $customerOnly [,,0]
 * @property Package $package (fk,?)
 * @property Product $product (fk,?)
 * @property int $position [,,0]
 * @property bool $deleted [,,false]
 * @property string $code [50]
 * @property bool $askPrice [,,false]
 * @property bool $askCount [,,false]
 * @property bool $askCustomName [,,false]
 * @property bool $stockable [,,false]
 * @property string $color [20](?)
 * @property int $type [,,0]
 * @property TaxRate $taxRate (fk)
 * @property ?array $widget (json,?)
 *
 * @defaultOrder position
 * @related StockChange $stockChanges
 **/

class Product extends BaseModel 
{
	const TypeProduct = 0,
		TypeCategory = 1,
		TypePage = 2;

	public $level = 0;

	public function getIsProduct()
	{
		return $this->type == self::TypeProduct;
	}

	public function getIsCategory()
	{
		return $this->type == self::TypeCategory;
	}

	public function getIsPage()
	{
		return $this->type == self::TypePage;
	}

	public static function defaults() 
	{
		return array(
			"enabled" => true,
			"position" => 0,
		);
	}
	
	public function getPrices($customerId) 
	{
		$res = array();
		// bezna cena
		$price = new ProductPrice;
		$price->price = $this->price;
		$price->type = ProductPrice::TRegular;
		$price->productId = $this->id;
		$res[] = $price;
		if ($customerId) {
			// Slevy, z kreditu
			/*foreach($this->context->database->query(
				'SELECT pp.fromCredit, pp.priceCredit, pp.priceDirect, p.creditLeft, p.id, p.utValidTo, pp.id AS packageProductId, pa.creditNegative '
				.'FROM packageProduct pp '
				.'LEFT JOIN prepaid p ON p.packageId = pp.packageId '
				.'LEFT JOIN package pa ON pa.id = p.packageId '
				.'WHERE pp.productId = ? AND p.customerId = ? AND (pp.fromCredit = 1 OR priceDirect IS NOT NULL) '
				.'AND p.utValidFrom IS NOT NULL AND (p.utValidTo >= ? OR p.utValidTo IS NULL)',
				$this->id, $customerId, time()) as $row
			) {*/
			foreach(self::repository()->newSqlQuery()
				->c(
					'SELECT pp.fromCredit, pp.priceCredit, pp.priceDirect, p.creditCLLeft, p.creditCALeft, p.id, p.utValidTo, pp.id AS packageProductId, pa.creditNegative 
					FROM packageProduct pp 
					LEFT JOIN prepaid p ON p.packageId = pp.packageId 
					LEFT JOIN package pa ON pa.id = p.packageId 
					WHERE pp.productId = ')->v($this->id)->c(' AND p.customerId = ')->v($customerId)->c(' AND (pp.fromCredit = 1 OR priceDirect IS NOT NULL) 
					AND p.utValidFrom IS NOT NULL AND (p.utValidTo >= ')->v(time())->c(' OR p.utValidTo IS NULL)')
				->exec() as $row
			) {
				$priceCredit = ($row->priceCredit !== null)
					? $row->priceCredit
					: $this->price;
				// Glob::varbar($row->creditLeft.' >= '.$row->priceCredit);
				// je mozne koupit z kreditu AND (je dostatek kreditu OR muze jin na sekyru)
				if ($row->fromCredit && ((($row->creditCLLeft + $row->creditCALeft) >= $priceCredit) || ($row->creditNegative))) {
					$price = new ProductPrice();
					$price->price = $priceCredit;
					$price->type = (($row->creditCLLeft + $row->creditCALeft) >= $priceCredit) ? ProductPrice::TPCredit : ProductPrice::TPCreditNegative;
					$price->prepaidId = $row->id;
					$price->productId = $this->id;
					$price->packageProductId = $row->packageProductId;
					$res[] = $price;
				}
				if ($row->priceDirect !== null) {
					$price = new ProductPrice();
					$price->price = $row->priceDirect;
					$price->type = ProductPrice::TPDiscount;
					$price->prepaidId = $row->id;
					$price->productId = $this->id;
					$price->packageProductId = $row->packageProductId;
					$res[] = $price;
				}
			}
			// Z pernamentky
			foreach(self::repository()->newSqlQuery()->c(
				'SELECT p.id, p.utValidFrom, p.utValidTo, ppp.amountLeft, p.customerId, pp.productId, pp.id AS packageProductId '
				.'FROM prepaidPackageProduct ppp '
				.'LEFT JOIN prepaid p ON p.id = ppp.prepaidId '
				.'LEFT JOIN packageProduct pp ON pp.id = ppp.packageProductId '
				.'WHERE pp.productId=')->v($this->id)->c(' AND p.customerId=')->v($customerId)->c(' '
				.'AND p.utValidFrom IS NOT NULL AND (p.utValidTo >= ')->v(time())->c(' OR p.utValidTo IS NULL) AND ppp.amountLeft > 0'
				)->exec() as $row
			) {
				$price = new ProductPrice();
				$price->amountLeft = $row->amountLeft;
				$price->type = ProductPrice::TPackage;
				$price->prepaidId = $row->id;
				$price->productId = $this->id;
				$price->packageProductId = $row->packageProductId;
				$res[] = $price;
			}
		}
		
		// Nejlevnejsi ze bezne ceny a slev
		$minPrice = null;
		foreach($res as $item) if ($item->type == ProductPrice::TRegular || $item->type == ProductPrice::TPDiscount) 
		{
			$item->hidden = true;
			if ($minPrice === null || $minPrice > $item->price) {
				$minPrice = $item->price;
			}
		}
		foreach($res as $item) if ($item->type == ProductPrice::TRegular || $item->type == ProductPrice::TPDiscount) 
		{
			if ($minPrice == $item->price) {
				$item->hidden = false;
				break;
			}
		}
		
		return $res;
	}

	public function getLastStockChange()
	{
		return $this->stockable
			? $this->stockChanges->where('isActual', true)->fetch()
			: null;
	}

	public function getStockAmount()
	{
		if (!$this->stockable) return null;
		$lsch = $this->lastStockChange;
		return $lsch ? $lsch->currentAmount : 0;			
	}

	// Repo
	public static function getByParent($onlyCategories = false, $excludeId = null) 
	{
		$ret = array();
		$items = self::findBy(['code' => '']);
		if ($onlyCategories) $items->where("type != ", self::TypeProduct);
		if ($excludeId) $items->where("id != ?", $excludeId);
			
		foreach($items->order("position") as $item) {
			$ret[$item->productId][] = $item;
		}
		
		return $ret;
	}
	
	public static function getIndented() 
	{
		$data = self::getByParent();
		$res = array();
		self::indentNode($data, $res);
		
		return $res;
	}
	
	public static function getCategoriesForSelect($excludeId = null) 
	{
		$data = self::getByParent(true, $excludeId);
		$res = array();
		self::indentNode($data, $res);
		$res2 = array("root" => "Bez kategorie");
		foreach($res as $item) {
			$res2[$item->id] = str_repeat("\xC2\xA0", $item->level) . $item->name;
		}
		
		return $res2;
	}
	
	protected static function indentNode(&$data, &$res, $parent = null) 
	{
		static $level = 0;
		if (isset($data[$parent])) foreach($data[$parent] as $item) {
			$item->level = $level;
			$res[] = $item;
			if (isset($data[$item->id])) {
				$level++;
				self::indentNode($data, $res, $item->id);
				$level--;
			}
		}
	}
	
	public static function getByCode($code) 
	{
		return self::getBy(['code' => $code]);
	}

	public function toArray($cols = NULL, $fkObjects = false, $prefix = '')
	{
		$data = parent::toArray($cols, $fkObjects, $prefix);
		$data['colorIsSet'] = !!$this->color;
		return $data;
	}

	public function getCssStyles()
	{
		return $this->color 
			// ? 'background: '.$this->color
			? 'background: '.$this->color.'; background: -webkit-linear-gradient(top, '.$this->color.' 0%,#ffffff 45%,#ffffff 65%,'.$this->color.' 100%);'
				.'box-shadow: inset 0 0 30px '.$this->color.';'
			: null;
	}

	public function getNameOnReceipt(?int $maxLength = null)
	{
		return $maxLength 
			? substr($this->name, 0, $maxLength)
			: $this->name;
	}
}

class ProductPrice extends NObject
{
	const TRegular = 1;
	const TPDiscount = 2;
	const TPCredit = 3;
	const TPackage = 4;
	const TPCreditNegative = 5;

	public $type;
	
	public $price;

	public $priceCachless;
	
	public $amountLeft;
	
	public $prepaidId;
	
	public $hidden;
	
	public $productId;
	
	public $packageProductId;
	
	public $customName;
	
	public $bonusCredit;

	static $typeNames = array(
		self::TRegular => array('Běžná cena', 'Běžná'),
		self::TPDiscount => array('Sleva za pernamentku', 'Sleva'),
		self::TPCredit => array('Cena z kreditu na pernamentce', 'Z kreditu'),
		self::TPackage => array('Počet zbývajících kusů na pernamentce', 'Pernamentka'),
		self::TPCreditNegative => array('Cena z kreditu na pernamentce', 'Na sekyru'),
	);
	
	public function getTypeName($short = false) {
		if ($short) {
			return self::$typeNames[$this->type][1];
		} else {
			return self::$typeNames[$this->type][1]
				.($this->prepaidId 
					? ': '.$this->prepaid->name
					: '');
		}
	}
	public function getTypeShortName() {
		return $this->getTypeName(true);
	}
	
	public function getPrepaid() {
		return Prepaid::get($this->prepaidId);
	}
	
	public function getCode() {
		return implode('-', array($this->productId, $this->type, $this->prepaidId, $this->packageProductId));
	}
	
	public function setCode($code) {
		// 'changeCredit-'.$value->id.'-'.$value->amountLeft
		if (substr($code, 0, 12) == 'changeCredit') {
			list($tmp, $this->prepaidId, $this->price) = explode('-', $code);
			$this->type = self::TRegular;
			// dump(Glob::context()->products->getByCode("creditChange"));
			$this->productId = Product::getBy(["code" => "creditChange"])->id;
		} else {
			list($this->productId, $this->type, $this->prepaidId, $this->packageProductId) = explode('-', $code);
			$this->prepaidId = $this->prepaidId ? $this->prepaidId : null;
			$this->packageProductId = $this->packageProductId ? $this->packageProductId : null;
			// Glob::varbar($this);
			switch ($this->type) {
				case ProductPrice::TRegular:
					$this->price = $this->product->price;
					break;
				case ProductPrice::TPDiscount:
					$this->price = $this->packageProduct->priceDirect;
					break;
				case ProductPrice::TPCredit:
				case ProductPrice::TPCreditNegative:
					$this->price = ($this->packageProduct->priceCredit !== null)
						? $this->packageProduct->priceCredit
						: $this->product->price;
					break;
				case ProductPrice::TPackage:
					break;
			}
		}
	}

	public function getProduct() 
	{
		return Product::get($this->productId);
	}
	
	public function getPackageProduct() 
	{
		return PackageProduct::get($this->packageProductId);
	}
	
	public function __construct($code = null) 
	{
		if ($code != null)  $this->code = $code;
	}
}
