<?php

namespace App;

use App\Service\EETSender;
use Nette\Utils\DateTime;

/**
@dbTable
@property autoIncrement $id
@property DateTime $dtCreate
@property ?DateTime $dtSend
@property int $counterValue
@property CachRegister $cachRegister (fk)
@property double $totalCache
@property double $totalCreditCA
@property double $totalCreditCL
@property string $state
@property ?string $pkp [500]
@property ?string $bkp [50]
@property ?string $fik [50]
@property ?string $errorMessage

@related ReceiptTaxItem $receiptTaxItems
**/
class Receipt extends BaseModel
{
	const State_new = "new";

	const State_sent = "sent";
	
	const State_sending = "sending";

	const State_failed = "failed";

	const State_empty = "empty";

	const stateNames = [
		self::State_new => "Neodeslaná",
		self::State_sent => "Odeslaná",
		self::State_sending => "Odesílá se",
		self::State_failed => "Nepovedlo se odeslat",
		self::State_empty => "Prázdná",
	];

	/**
	 * Připraví účtenku:
	 * 	- nastaví datum vytvoření
	 * 	- vytvoří položky podle typu dph
	 * 	- nastaví cekovou částku
	 * 	- v případě že je částka > 0
	 * 		- získá pořadové číslo
	 * Požadavky:
	 * 	- připojené ProductSell
	 * 
	 * @param ProductSell[] $productSells
	 */
	public function prepare($productSells)
	{
        return;
		$sumsByTaxRate = [];
		$totalCache = 0;
		$totalCreditCA = 0;
		$totalCreditCL = 0;
		/** @var ProductSell[][] $linkedProductSells */
		$linkedProductSells = [];
		$mustSend = false;
		foreach($productSells as $productSell)
		{
			/** @var ProductSell $productSell */
			// if ($productSell->priceDirectpaid || $productSell->priceCreditCA || $productSell->priceCreditCL) continue;
			$taxRateId = $productSell->product->taxRateId;
			if (!isset($sumsByTaxRate[$taxRateId])) $sumsByTaxRate[$taxRateId] = 0;
			$sumsByTaxRate[$taxRateId] += $productSell->priceDirectpaid + $productSell->priceCreditCA;
			$totalCache += $productSell->priceDirectpaid;
			$totalCreditCA += $productSell->priceCreditCA;
			$totalCreditCL += $productSell->priceCreditCL;
			if (!isset($linkedProductSells[$taxRateId])) $linkedProductSells[$taxRateId] = [];
			$linkedProductSells[$taxRateId][] = $productSell;
			if ($productSell->priceDirectpaid != 0 || $productSell->priceCreditCA != 0) $mustSend = true;
		}

		// Bude se vytvářet účtenka
		$this->dtCreate = new DateTime();
		$this->totalCache = $totalCache;
		$this->totalCreditCA = $totalCreditCA;
		$this->totalCreditCL = $totalCreditCL;
		$this->counterValue = -1;
		if ($mustSend)
		{
			$this->counterValue = $this->cachRegister->getNextCounterValue();
			$this->state = self::State_new;
			$this->save();
			$trToRti = [];
			foreach($sumsByTaxRate as $taxRateId => $amount)
			{
				$rti = new ReceiptTaxItem();
				$rti->receiptId = $this->id;
				$rti->taxRateId = $taxRateId;
				// částka je s DPH, dopočítá se částka bez DPH a DPH
				$rti->priceWT = $amount;
				$rti->priceNT = ($rti->priceWT / ($rti->taxRate->rate / 100 + 1));
				$rti->priceTax = $rti->priceWT - $rti->priceNT;
	
				$rti->save();

				$trToRti[$rti->taxRateId] = $rti->id;
			}

			// Pripoj položky účtenky
			foreach($linkedProductSells as $taxRateId => $pss)
			foreach($pss as $ps)
			{
				$ps->receiptTaxItemId = $trToRti[$taxRateId];
				$ps->save();
			}
		}

		else $this->state = self::State_empty;
		// dump($this->toArray());
	}

	public function send()
	{
		/** @var \App\Service\EETSender $eetSender */
		$eetSender = Glob::context()->getService("EETSender");
		$eetSender->send($this);
		$this->save();
	}

	public function getStateName()
	{
		return self::stateNames[$this->state];
	}

	public function getUid()
	{
		return $this->cachRegister->company->name."-"
			.$this->cachRegister->workshopUID."-"
			.$this->cachRegister->cashRegisterUID."-"
			.$this->counterValue;
	}

	public function getProductSells(bool $cachOnly = false)
	{
		return $cachOnly
			? ProductSell::findBy(["" => $this->id])
			: ProductSell::repository()->newSqlQuery()
				->c("SELECT DISTINCT ps.*
					FROM receiptTaxItem rti
					LEFT JOIN productSell ps1 ON ps1.receiptTaxItemId = rti.id
					LEFT JOIN directpaid d ON ps1.directpaidId = d.id
					LEFT JOIN productSell ps ON ps.directpaidId = d.id
					WHERE rti.receiptId = ?
					ORDER BY utSell", $this->id);
	}
}