<?php

namespace App;

use Nette;
use Nette\Security\Passwords;
use App\User;


/**
 * Users management.
 */
class UserManager implements Nette\Security\IAuthenticator
{
	use Nette\SmartObject;

	/* const
		TABLE_NAME = 'users',
		COLUMN_ID = 'id',
		COLUMN_NAME = 'username',
		COLUMN_PASSWORD_HASH = 'password',
		COLUMN_EMAIL = 'email',
		COLUMN_ROLE = 'role'; */


	/** @var Nette\Database\Context */
	private $database;

    private Passwords $password;

    public function __construct(Nette\Database\Context $database)
	{
		$this->database = $database;
        $this->password = new Passwords();
	}


	/**
	 * Performs an authentication.
	 * @return Nette\Security\Identity
	 * @throws Nette\Security\AuthenticationException
	 */
	public function authenticate(array $credentials): Nette\Security\IIdentity
	{
		list($username, $password) = $credentials;

		/** @var App\User **/
		$user = User::getBy(['username' => $username]);

		if (!$user) {
			throw new Nette\Security\AuthenticationException('The username is incorrect.', self::IDENTITY_NOT_FOUND);

		} elseif (/* false && */ !$this->password->verify($password, $user->passwordHash)) {
			// dump($this->password->hash($password));
			throw new Nette\Security\AuthenticationException('The password is incorrect. ', self::INVALID_CREDENTIAL);


		} elseif ($this->password->needsRehash($user->passwordHash)) {
			$user->passwordHash = $this->password->hash($password);
			$user->save();
			/* $row->update([
				self::COLUMN_PASSWORD_HASH => $this->password->hash($password),
			]); */
		}

		$arr = $user->toArray();
		unset($arr['passwordHash']);

		return new Nette\Security\Identity($user->id, null, $arr);
	}


	/**
	 * Adds new user.
	 * @param  string
	 * @param  string
	 * @param  string
	 * @return void
	 * @throws DuplicateNameException
	 */
	public function add($username, $email, $password)
	{
		try {
			$this->database->table(self::TABLE_NAME)->insert([
				self::COLUMN_NAME => $username,
				self::COLUMN_PASSWORD_HASH => $this->password->hash($password),
				self::COLUMN_EMAIL => $email,
			]);
		} catch (Nette\Database\UniqueConstraintViolationException $e) {
			throw new DuplicateNameException;
		}
	}

}



class DuplicateNameException extends \Exception
{}
