<?php

namespace App\Presenters;

use App\Glob;
use App\Service\SolarConfig;
use Nette;

/**
 * Base presenter for all application presenters.
	@property \Murdej\Menu $menu 
**/
abstract class BasePresenter extends \Nette\Application\UI\Presenter
{
	public function getMenu() {
		return $this->getComponent('menu');
	}
	
	public function createComponentMenu() {
		$menu = new \Murdej\Menu;
		
		if ($this->user->isLoggedIn()) {
			$nCustomer = $menu->rootNode->add(array('name' => 'Pokladna', 'link' => 'Cashbox:', 'id' => 'Cashbox'));

			/*$nCustomer = $menu->rootNode->add(array('name' => 'Zákazníci', 'link' => 'customer:list', 'id' => 'customer'));
			$nCustomer->add(array('name' => 'Seznam', 'link' => 'customer:list', 'id' => 'customerList'));
			$nCustomer->add(array('name' => 'Přidat', 'link' => array('customer:list', 'do' => 'edit', 'id' => ''), 'id' => 'customerNew')); */

			$nAdmin = $menu->rootNode->add(array('name' => 'Administrace', 'link' => 'Admin:default', 'id' => 'admin'));
			$nAdmin->add(array('name' => 'Produkty', 'link' => 'Admin:product', 'id' => 'Admin:product'));
			$nAdmin->add(array('name' => 'Pernamentky', 'link' => 'Admin:package', 'id' => 'Admin:package'));
			$nAdmin->add(array('name' => 'Zákazníci', 'link' => 'Admin:customer', 'id' => 'Admin:customer'));
			$nAdmin->add(array('name' => 'Uživatelé', 'link' => 'Admin:user', 'id' => 'Admin:user'));
			$nAdmin->add(array('name' => 'Součty', 'link' => 'Admin:sums', 'id' => 'Admin:sums'));
			$nAdmin->add(array('name' => 'Vytvořit zálohu', 'link' => 'Admin:export', 'id' => 'Admin:export'));
			if (Glob::logedUser()->checkPerm('admin.adv-sums')) $nAdmin->add(array('name' => 'Úprava součtů', 'link' => 'Admin:advsums', 'id' => 'Admin:advsums'));
			
			$nStock = $menu->rootNode->add(array('name' => 'Sklad', 'link' => 'Stock:default', 'id' => 'Stock:default'));
			$nStock->add(array('name' => 'Stav', 'link' => 'Stock:default', 'id' => 'Stock:default2'));
			$nStock->add(array('name' => 'Naskladnit', 'link' => 'Stock:stockFill', 'id' => 'Stock:stockFill'));
			$nStock->add(array('name' => 'Historie skladu', 'link' => 'Stock:stockFillHistory', 'id' => 'Stock:stockFillHistory'));

			$menu->rootNode->add(array('name' => 'Odhlásit', 'link' => 'Sign:out', 'id' => 'Sign:out'));
		};
		
		return $menu;
	}      

	protected function createTemplate($class = NULL, $latteFactory = NULL) : Nette\Application\UI\ITemplate
	{
		$template = parent::createTemplate($class, $latteFactory);
		Glob::setupHeplers($template);
		return $template;
	}
	
	protected function startup() 
	{
		$noLogged = [
			'Sign:*',
			'Error:*',
			'Admin:customerExport',
            'Solarko:*'
		];
		$pa1 = $this->name . ":" . $this->getAction(false);
		$pa2 = $this->name . ":*";
		$needLogin = !(in_array($pa2, $noLogged) || in_array($pa1, $noLogged));
		// echo "( $pa1, $pa2 : " . ($needLogin ? "T" : "F") . ")";
		parent::startup();
		$this->template->customer = '';
		$this->template->fixScroll = '';
		// echo $this->name;
		if ($this->user->isLoggedIn()) {
			$this->template->user = Glob::logedUser()->name;
		} else if (!$needLogin) {
			$this->template->user = "Nepřihlášen";
			// echo ' login form ';
			// exit();
		} else {
			// echo ' redir ';
			// exit();
			$this->redirect('Sign:in');
		}

        if ($this->context->hasService("solarConfig")) $this->template->solarConfig = $this->context->getByName("solarConfig");
        // dump($this->template->solarConfig);

		// Kontrola opravneni, jen mimo Sign
		if ($needLogin) 
		{
			// $this->flashMessage($this->name.":".$this->action);
			if (!Glob::logedUser()->role->checkPagePermission($this->name, $this->action)) {
				$this->flashMessage('Nemáte oprávnění přístupu do modulu '.$this->name.':'.$this->action);
				$this->redirect('Cashbox:');
			}
		}
	}
	
	protected function beforeRender() {
		if (!$this->menu->selected) $this->menu->selectByUrl($this->link('this'));
	}
	
	public function flashMessages($messages) {
		foreach($messages as $message) $this->flashMessage($message);
	}
}
