<?php

namespace App\Presenters;

use Nette\Application\UI;
use Nette\Security\AuthenticationException;

/**
 * Sign in/out presenters.
 @property \Murdej\ActiveRow\DBSelect $users
 */
class SignPresenter extends BasePresenter
{

	/**
	 * Sign-in form factory.
	 * @return Nette\Application\UI\Form
	 */
	protected function createComponentSignInForm()
	{
		$form = new UI\Form;
		$form->addText('username', 'Uživatelské jméno')
			->setRequired('Zadejte uživatelské jméno.');

		$form->addPassword('password', 'Heslo')
			->setRequired('Zadejte heslo.');

		$form->addSubmit('send', 'Přihlásit');

		// call method signInFormSucceeded() on success
		$form->onSuccess[] = [ $this, 'signInFormSucceeded' ];
		return $form;
	}



	public function signInFormSucceeded($form)
	{
		$values = $form->getValues();
		$this->getUser()->setExpiration('+ 60 minutes', TRUE);

		try 
		{
			$this->getUser()->login($values->username, $values->password);
		} 
		catch (AuthenticationException $e) 
		{
			$form->addError($e->getMessage());
			return;
		}

		$this->redirect('Cashbox:');
	}



	public function actionOut()
	{
		$this->getUser()->logout();
		$this->flashMessage('Byl jste odhlášen.');
		$this->redirect('in');
	}

	public function getUsers() 
	{
		return \App\User::findBy(['deleted' => false])->order('name');
	}
}
