<?php

namespace Murdej\Forms;

use Nette\Forms\Form;
use Nette\Forms\Container as FormContainer;

/**
* Selectize input control.
*
* @author     Murdej
* @package    Nette\Extras
*/
class SelectizeMulti extends \Nette\Forms\Controls\BaseControl
{

	public function __construct($label, $items, $complex)
	{
		parent::__construct($label);
		$this->setAttribute('class', 'selectizeMulti');
		$this->setItems($items, $complex);
	}

	protected $value = [];

	public $delimiter = '#';

	protected $items = [];

	public $isNullable = true;

	public function setItems($items, $complex = false)
	{
		$this->items = [];
		foreach($items as $k => $v)
		{
			if ($complex)
				$value = $v;
			else
				$value = [
					'id' => $k,
					'name' => $v
				];
			$this->items[] = $value;
		}
	}

	/**
	* Returns control's value.
	* @return mixed 
	*/
	public function getValue()
	{
		return (!$this->value && $this->isNullable) ? null : $this->value;
	}


	/**
	* Sets control's value.
	* @param  string
	* @return void
	*/
	public function setValue($value)
	{
		$this->value = $value;
	}

	public function loadHttpData(): void
	{
		$values = [];
		$rawData = $this->getHttpData(Form::DATA_LINE);
		if ($rawData) foreach (explode($this->delimiter, $rawData) as $value) 
		{
			if ($value != '') $values[] = $value;
		}
		$this->value = $values;
	}

	protected function getEncodedValue()
	{
		return $this->value ? implode($this->delimiter, $this->value) : '';
	}

	public function getControl()
	{
		$el = clone $this->control;
		return $el->addAttributes([
			'id' => $this->getHtmlId(),
			'type' => 'text',
			'name' => $this->getHtmlName(),
			'class' => array('form-control text selectizeMulti'),
			'data-items' => $this->items,
			'data-options' => [ 
				'delimiter' => $this->delimiter
			],
			'value' => $this->getEncodedValue()
		]);
	}


	static function register() 
	{
		FormContainer::extensionMethod('addSelectizeMulti', array(__CLASS__, 'addSelectizeMulti')); // v PHP 5.3
	}

	static function addSelectizeMulti($_this, $name, $label, $items = [], $complex = false)
	{
		return $_this[$name] = new SelectizeMulti($label, $items, $complex);
	}
}
