<?php

namespace Murdej\Forms;

use Nette\Forms\Form;
use Nette\Forms\Container as FormContainer;
use Nette\Utils\Html;
use Nette;
/**
* Yer No (none) radiolist input control.
*
* @author     Murdej
* @package    Nette\Extras
*
* @property mixed $value
* @property mixed $prompt
*/
class YesNo extends \Nette\Forms\Controls\BaseControl
{
	use \Nette\SmartObject;

	public $itemLabels = [
		'true' => 'Ano',
		'false' => 'Ne',
		'null' => 'Nezáleží'
	];

	protected $value;

	public function __construct($label, $trueLabel = null, $falseLabel = null, $nullLabel = null)
	{
		$this->itemLabels['true'] = $trueLabel === null ? 'Ano' : $trueLabel;
		$this->itemLabels['false'] = $falseLabel === null ? 'Ne' : $falseLabel;
		$this->itemLabels['null'] = $nullLabel === null ? 'Nezáleží' : $nullLabel;

		parent::__construct($label);
	}

	/**
	 * Generates control's HTML element.
	 * @return Html
	 */
	public function getControl()
	{
		$container = Html::el();
		$inputTemplate = parent::getControl();
		$strVal = $this->getStringValue();
		foreach($this->getItems() as $v => $n)
		{
			$input = clone $inputTemplate;
			$input->setAttribute('type', 'radio');
			$input->setAttribute('name', $this->getName())
				->setValue($v);
			$input->id = $input->id . '-' . $v;
			if ($strVal == $v) $input->setChecked(true);
			$label = Html::el('label');
			$label->setFor($input->id)
				->addHtml($input)
				->addText($n);
			$itemContainer = Html::el('div')
				->setClass('rbl-horizontal-item radio')
				->addHtml($label);
			$container->addHtml($itemContainer);
		}

		return $container;
	}

	public function getStringValue()
	{
		switch(true)
		{
			case $this->value === true: return 'true';
			case $this->value === false: return 'false';
			case $this->value === null: return 'null';
		}
		return null;
	}

	public function getItems()
	{
		$res = [];
		foreach($this->itemLabels as $v => $n)
		{
			if ($n) $res[$v] = $n;
		}

		return $res;
	}

	/**
	* Loads HTTP data.
	* @return void
	*/
	public function loadHttpData(): void
	{
		$this->setValue($this->getHttpData(Nette\Forms\Form::DATA_TEXT));
	}

	public function getValue()
	{
		return $this->value;
	}

	public function setValue($value)
	{
		if ($value === null || $value === true || $value === false) 
		{

		}
		else switch($value)
		{
			case 'true':
			case '1':
				$value = true; 
				break;
			case 'false': 
			case '0': 
				$value = false; 
				break;
			default: $value = null; break;
		}
		$this->value = $value;
	}

	static function register() 
	{
		FormContainer::extensionMethod('addYesNo', function($form, $name, $label, $trueLabel = null, $falseLabel = null, $nullLabel = null)
		{
			return $form[$name] = new YesNo($label, $trueLabel, $falseLabel, $nullLabel);
		});
	}
}
