<?php

namespace Murdej\Forms;

use Nette\Forms\Form;
use Nette\Forms\Container as FormContainer;

// require_once LIBS_DIR . '/Nette/Forms/Controls/TextInput.php';

/**
 * DatePicker input control.
 *
 * @author     Murdej, Tomáš Kraina, Roman Sklenář
 * @package    Nette\Extras
 */
class DateInput extends \Nette\Forms\Controls\TextInput		
{

	var $profile = null;
	
	var $options = array();
	
	/**
	 * @param  string  label
	 * @param  int  width of the control
	 * @param  int  maximum number of characters the user may enter
	 */
	public function __construct($label, $profile = 'near', $cols = 15, $maxLenght = 10, $minDate = null, $maxDate = null)
	{
		parent::__construct($label, $cols, $maxLenght);
		$this->profile = $profile;
		$this->addCondition(Form::FILLED)
			->addRule(Form::PATTERN, 'Zadejte datum ve formatu den.mesic.rok', '^(([0-9]{1,2}\\.[0-9]{1,2}\\.[0-9]{4})|(-?[0-9]+))$'); //'^[0123]?[0-9].[01]?[0-9].[0-9]{4}$'
	}


	/**
	 * Returns control's value.
	 * @return mixed 
	 */
	public function getValue()
	{
		// echo "getValue(): ".$this->value.' ';
		if (strlen($this->value)) {
			list($d, $m, $y) = explode('.', preg_replace('~([[:space:]])~', '', $this->value));
			
			return mktime (0, 0, 0, (int)$m, (int)$d, (int)$y);
		}
		
		return null;
	}


	/**
	 * Sets control's value.
	 * @param  string
	 * @return void
	 */
	public function setValue($value)
	{
		// echo "setValue(): ".$value.' ';
		parent::setValue(is_numeric($value) ? date("d.m.Y", $value) : $value);
	}


	/**
	 * Generates control's HTML element.
	 * @return Html
	 */
	public function getControl() : \Nette\Utils\Html
	{		
		$control = parent::getControl();
		$control->class = 'dateinput';
		$control->attrs['data-profile'] = $this->profile;
		$opt = $this->options;
		if (isset($opt['minDate']) && (get_class($opt['minDate']) == 'DateTime')) $opt['minDate'] = $opt['minDate']->format('d-m-Y'); //format('Y-m-d');
		if (isset($opt['minDate']) && (is_int($opt['minDate']))) $opt['minDate'] = date('Y-m-d', $opt['minDate']);
		if (isset($opt['maxDate']) && (get_class($opt['maxDate'])) == 'DateTime') $opt['maxDate'] = $opt['maxDate']->format('d-m-Y'); //format('Y-m-d');
		if (isset($opt['maxDate']) && (is_int($opt['maxDate']))) $opt['maxDate'] = date('Y-m-d', $opt['maxDate']);
		$control->attrs['data-options'] = json_encode($opt);
		$control->autocomplete = "off";
		
		return $control;
	}
	
	static function registerNette() {
		// FormContainer::extensionMethod('addDateInput', callback(__CLASS__, 'addDateInput')); // v PHP 5.3
		FormContainer::extensionMethod('addDateInput', array(__CLASS__, 'addDateInput')); // v PHP 5.3
	}

	static function register() 
	{
		FormContainer::extensionMethod('addDateInput', array(__CLASS__, 'addDateInput')); // v PHP 5.3
	}


	static function addDateInput($_this, $name, $label, $profile = 'near', $cols = 12, $maxLength = 10)
	{
		// \Murdej\PageHead::addOnLoad('$(".dateinput").datepicker({ showButtonPanel: true });', 1, true);
		// \Nette\Diagnostics\Debugger::barDump(\Murdej\PageHead::getInstance());
		return $_this[$name] = new DateInput($label, $profile, $cols, $maxLength);
	}
}
