<?php

namespace Murdej\Forms;

use Nette\Forms\Form;
use Nette\Forms\Container as FormContainer;
use Nette\Utils\Html;
use Murdej\ProcI;
/*todo:
	+ Přejmenovat soubor
	+ Nové položky, (get|set)Value
	+ static::register add(Single|Multi)Selectize
	+ prompt
	+ composer
*/
/**
* Radiotree input control using fancytree.
*
* @author     Murdej
* @package    Nette\Extras
*
* @property array $widgetParams
* @property mixed $prompt
*/
class RadioTree extends \Nette\Forms\Controls\BaseControl
{
	use \Nette\SmartObject;

	public $rootItem = [];

	/** @var string */
	public $prompt = "Nevybráno";

	/** @var bool */
	public $compact = false;

	/** @var bool */
	public $multiple = false;

	/** @var ?bool */
	public $inModal = null;

	/** @var string */
	public $clearButtonText = null;

	/** @var string */
	public $expanded = [];

	/** @var Callable|string */
	public $lazyLoad = null;

	/** @var Callable|string */
	public $searchLoad = null;

	/** @var null|Callable */
	public $missingValueCallback = null;

	/**
	 * @param array|ListItem|null $rootItem
	 */
	public function __construct($label, /* ?ListItem */ $rootItem = null, bool $compact = false, bool $multiple = false)
	{
		parent::__construct($label);
		if (is_array($rootItem))
		{
			$this->rootItem = new ListItem();
			$this->rootItem->items = $rootItem;
		}
		else $this->rootItem = $rootItem ?: new ListItem();
		$this->compact = $compact;
		$this->multiple = $multiple;
	}

	/**
	* Generates control's HTML element.
	* @return Html
	*/
	public function getControl() : Html
	{
		// Check missing values
		if ($this->missingValueCallback)
		{
			$itemValues = ProcI::from($this->rootItem->getAllNodes())->map(".value")->toArray();
			if ($this->multiple)
			{
				if ($diff = array_diff($itemValues, $this->value))
				{
					($this->missingValueCallback)($diff, $this->rootItem);
				}
			}
			else
			{
				if (!in_array($this->value, $itemValues))
				{
					($this->missingValueCallback)([$this->value], $this->rootItem);
				}
			}
		}
		$input = clone parent::getControl();
		$input->setAttribute("mu-id", "input");
		$input->setAttribute(
			"value", 
			$this->multiple
				? json_encode($this->value)
				: $this->value
		);
		$input->setAttribute("type", "hidden");
		$el = Html::el('div')
			->setAttribute("mu-widget", "MurdejFormRadioTree")
			->setAttribute("mu-params", json_encode($this->widgetParams));
		$el->addHtml($input);

		return $el;
	}

	protected function getWidgetParams()
	{
		return [
			'items' => $this->itemToWItem($this->rootItem),
			'compact' => $this->compact,
			'prompt' => $this->prompt,
			'multiple' => $this->multiple,
			'clearButtonText' => $this->clearButtonText,
			'inModal' => $this->inModal,
			'expanded' => $this->expanded,
			"lazyUrl" => $this->lazyLoad,
			"searchUrl" => $this->searchLoad
		];
	}

	protected function itemToWItem(ListItem $li)
	{
		return $li->fancyTreeData(true);
		/*
		$item = [
			'title' => $li->title,
			'key' => $li->value,
		];
		if ($li->isGroup) $item['checkbox'] = false;
		if ($li->items) $item['children'] = array_map([$this, 'itemToWItem'], $li->items);
		if ($li->data) $item['data'] = $li->data;

		return $item; */
	}

	public function setPrompt(string $prompt)
	{
		$this->prompt = $prompt;
		return $this;
	}

	public function setClearButtonText(string $clearButtonText)
	{
		$this->clearButtonText = $clearButtonText;
		return $this;
	}

	static function register() 
	{
		FormContainer::extensionMethod("addRadioTree", function($form, $name, $label, $items = [], $compact = false, $multiple = false)
		{
			$form[$name] = new RadioTree($label, $items, $compact, $multiple);
			return $form[$name];
		});
		FormContainer::extensionMethod("addChecboxTree", function($form, $name, $label, $items = [], $compact = false, $multiple = true)
		{
			$form[$name] = new RadioTree($label, $items, $compact, $multiple);
			return $form[$name];
		});
	}

	public function loadHttpData(): void
	{
		$rawData = trim($this->getHttpData(Form::DATA_LINE));
		$this->value = $this->multiple
			? json_decode($rawData)
			: $rawData;
	}
}