<?php

namespace Murdej\Forms;

use App\Model\Utils;
use Nette;
use Nette\Forms\Container as FormContainer;

trait TChoiceControlMissingValueCallback
{
	/**
	 * @param null|function($values) : array
	 */
	public $missingValueCallback = null;

	/**
	 * Sets selected item (by key).
	 * @param  string|int
	 * @return static
	 * @internal
	 */
	public function setValue($value)
	{
		if (/*$this->checkAllowedValues && */$value !== null && !array_key_exists((string) $value, $this->items))
		{
			if ($this->missingValueCallback)
			{
				$cb = $this->missingValueCallback;
				$this->items += $cb([$value]);
			}
		}

		if (/*$this->checkAllowedValues && */$value !== null && !array_key_exists((string) $value, $this->items)) {
			$set = Nette\Utils\Strings::truncate(implode(', ', array_map(function ($s) { return var_export($s, true); }, array_keys($this->items))), 70, '...');
			throw new Nette\InvalidArgumentException("Value '$value' is out of allowed set [$set] in field '{$this->name}'.");
		}
		$this->value = $value === null ? null : key([(string) $value => null]);

		return $this;
	}


	/**
	 * Returns selected key.
	 * @return string|int
	 */
	public function getValue()
	{
		$missing = !array_key_exists($this->value, $this->items);
		if ($missing && $this->missingValueCallback)
		{
			// zkus načíst, pokud něco vrátí je to platná hodnota
			$cb = $this->missingValueCallback;
			return $cb([$this->value], true) ? $this->value : null;
		}
		$value = array_key_exists($this->value, $this->items) ? $this->value : null;
		return $value;
	}

	public static function register()
	{
		$cn = get_called_class();
		FormContainer::extensionMethod(
			'add'.Utils::classNameNNS($cn), 
			function($form, $name, ...$args) use ($cn)
			{
				return $form[$name] = new $cn(...$args);
			}
		);
	}
}
