<?php

use Nette\SmartObject;

class SolarController
{
    use SmartObject;

    /**
     * @var SolarStation[]
     */
    protected array $stations;

    public function __construct(
        public string $url,
        public Logger $logger
    )
    {
    }

    public function addStation(SolarStation $solarStation)
    {
        $this->stations[$solarStation->outNum] = $solarStation;
        $solarStation->controler = $this;
        $solarStation->onSetStatus = fn() => $this->updateStations();
    }

    public function setOutput(int $outNum, bool $status)
    {
        $this->logger->log("SET", "Output $outNum set to $status.");
        $res = file_get_contents($this->url . '/outs.cgi?out' . $outNum . '=' . (int)$status);
    }

    public function updateStations()
    {
        $res = file_get_contents($this->url . '/json/outputs_per.json');
        $jRes = json_decode($res);
        $bOut = $jRes->out;
        for($i = 0; $i < 6; $i++) {
            $state = $bOut % 2;
            $bOut = $bOut >> 1;

            if (isset($this->stations[$i])) {
                $this->stations[$i]->outStatus = $state;
                $this->logger->log("GET", "Output $i is $state.");
            }
        }

    }

    public function start(int $outNum): string
    {
        $station = $this->stations[$outNum] ?? null;
        if ($station) {
            if ($station->canStart()) {
                 $station->startTimer();
                 $this->logger->log("START", "On $outNum.");
                 return true;
            }
            $this->logger->log("START", "No planned $outNum.");
            return "Not planned";
        }
        $this->logger->log("START", "Not found $outNum.");
        return "Not found";
    }

    public function status(): array
    {
        return array_map(
            fn($station) => $station,
            $this->stations
        );
    }

    public function setStatus(int $outNum, string $requiredStatus, int|null $duration = null)
    {
        $station = $this->stations[$outNum] ?? null;
        if ($station) {
            return $station->requireOutStatus($requiredStatus, $duration);
        }
    }

    public function refresh()
    {
        foreach ($this->stations as $station) $station->refresh();
        $this->updateStations();
    }
}