<?php

use Nette\SmartObject;

class SolarStation implements JsonSerializable
{
    const RequiredOutStatus_On = "On";
    const RequiredOutStatus_Off = "Off";
    const RequiredOutStatus_WaitForStart = "WaitForStart";
    const RequiredOutStatus_OnTimer = "OnTimer";

    public string $requiredOutStatus = SolarStation::RequiredOutStatus_Off;

    public DateTime|null $stopDateTime = null;

    public int|null $duration = null;

    public function __construct(
        public int $outNum = 0,
        public string $inId = 'inpp1',
    )
    {
    }

    use SmartObject;

	public SolarController $controler;

	public bool $outStatus = false;

	/**
	 * 
	 */
	public $onSetStatus = null;

	public function requireOutStatus(string $status, int|null $duration = null): bool|string
	{
        $bStatus = null;
        $this->requiredOutStatus = $status;
        switch($status) {
            case self::RequiredOutStatus_Off:
                $bStatus = false;
                $this->duration = 0;
                break;
            case self::RequiredOutStatus_On:
                $bStatus = true;
                $this->duration = 0;
                break;
            case self::RequiredOutStatus_OnTimer:
                $this->duration = $duration;
                $this->startTimer();
                break;
            case self::RequiredOutStatus_WaitForStart:
                $this->duration = $duration;
                break;
            default:
                return "Unknown status '$status'.";
        }
        if ($bStatus !== null) $this->controler->setOutput($this->outNum, $bStatus);
		$cb = $this->onSetStatus;
		if ($cb) $cb($this);
		return $this->outStatus;
	}

    public function canStart(): bool
    {
        return $this->requiredOutStatus == self::RequiredOutStatus_WaitForStart;
    }

    public function jsonSerialize(): array
    {
        return [
            'outNum' => $this->outNum,
            'outStatus' => $this->outStatus,
            'requiredOutStatus' => $this->requiredOutStatus,
            'stopDateTime' => $this->stopDateTime?->format('Y-m-d\\TH:i:s'),
            'duration' => $this->duration,
            'left' => $this->requiredOutStatus === SolarStation::RequiredOutStatus_OnTimer
                ? max(0, $this->stopDateTime->getTimestamp() - (new DateTime())->getTimestamp())
                : null,
            'valid' => $this->isValid()
        ];
    }

    public function startTimer()
    {
        $this->stopDateTime = (new DateTime('now'))->modify("+" . $this->duration . " seconds");
        $this->requiredOutStatus = self::RequiredOutStatus_OnTimer;
        $this->controler->setOutput($this->outNum, true);
    }

    public function checkTimer()
    {
        if ($this->requiredOutStatus == SolarStation::RequiredOutStatus_OnTimer && $this->stopDateTime <= new DateTime('now'))
        {
            $this->controler->setOutput($this->outNum, false);
            $this->requiredOutStatus = SolarStation::RequiredOutStatus_Off;
            $this->stopDateTime = null;
            $this->duration = null;
        }
    }

    public function refresh()
    {
        $this->controler->logger->log("REFRESH", "$this->outStatus. $this->requiredOutStatus");
        $this->checkTimer();
        if (in_array($this->requiredOutStatus, [SolarStation::RequiredOutStatus_On, SolarStation::RequiredOutStatus_OnTimer])) $this->controler->setOutput($this->outNum, true);
    }

    public function isValid()
    {
        return ($this->outStatus && in_array($this->requiredOutStatus, [SolarStation::RequiredOutStatus_On, SolarStation::RequiredOutStatus_OnTimer]))
            || (!$this->outStatus && in_array($this->requiredOutStatus, [SolarStation::RequiredOutStatus_Off, SolarStation::RequiredOutStatus_WaitForStart]));
    }
}
